package com.ai.bss.es.esmanage.service.impl;

import com.ai.abc.jpa.elastic.model.ElasticPage;
import com.ai.abc.jpa.elastic.model.PageInfo;
import com.ai.abc.jpa.elastic.model.TimeInfo;
import com.ai.abc.jpa.elastic.repository.ElasticEntityRepository;
import com.ai.abc.jpa.elastic.repository.impl.ElasticEntityRepositoryImpl;
import com.ai.bss.es.esmanage.service.EsService;
import com.ai.bss.es.esmanage.utils.RequestEntity;
import com.ai.bss.es.esmanage.utils.ResponseResult;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.TermsQueryBuilder;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;


@Slf4j
@Service
public class EsServiceImpl implements EsService {


    @Value("${es.server:47.105.160.21:10020}")
    String esServer ;


    ElasticEntityRepository elasticEntityRepository = null;

    private ElasticEntityRepository getElasticEntityRepository(){
        if(elasticEntityRepository==null){
            elasticEntityRepository = new ElasticEntityRepositoryImpl();
            elasticEntityRepository.initServer(esServer);
        }
        return elasticEntityRepository;
    }


    @Override
    public ResponseResult executeEsEntity(RequestEntity requestEntity) {

        if(requestEntity==null){
            return null;
        }

        String methods = requestEntity.getMethods();

        if("SAVE".equals(methods)){
            saveEsEntity(requestEntity);
            return ResponseResult.sucess();
        }else{
            return findEsEntityList(requestEntity);
        }

    }

    private void saveEsEntity(RequestEntity requestEntity){

        getElasticEntityRepository().purePersist(requestEntity.getTableName(), "", requestEntity.getEntity());
    }

    private ResponseResult findEsEntityList(RequestEntity requestEntity) {

        PageInfo pageInfo = new PageInfo();
        int pageSize = requestEntity.getPageSize() == null ? 0 : requestEntity.getPageSize();
        int startNum = requestEntity.getStartNumber() == null ? 0 : requestEntity.getStartNumber()*pageSize;
        pageInfo.setStartNumber(startNum);
        pageInfo.setPageSize(pageSize);
        Map entityMap = null;
        if (!StringUtils.isEmpty(requestEntity.getEntity())) {

            String entityStr = requestEntity.getEntity();
            entityMap = JSONObject.parseObject(entityStr, new TypeReference<Map<String, Object>>() {});
        }


        Map fieldMap = null;
        String entityKey;
        if (entityMap != null && !entityMap.isEmpty()) {
            fieldMap = new HashMap();
            Iterator var10 = entityMap.entrySet().iterator();

            while(var10.hasNext()) {
                Map.Entry<String, Object> entry = (Map.Entry)var10.next();
                entityKey = (String)entry.getKey();
                if (entry.getValue() != null) {
                    TermsQueryBuilder termsQueryBuilder = null;
                    if (entry.getValue().getClass() == JSONArray.class) {
//                        this.log.debug("mapKey=" + sortType + ",entry.getValue()=" + entry.getValue());
//                        termsQueryBuilder = QueryBuilders.termsQuery(sortType, (ArrayList)entry.getValue());
                        fieldMap.put(entityKey,JSONArrayToArrayList((JSONArray)entry.getValue()));
                    } else {
                        String mapValue = entry.getValue().toString();
//                        termsQueryBuilder = QueryBuilders.termsQuery(sortType, new String[]{mapValue});
                        fieldMap.put(entityKey,mapValue);
                    }

                }
//                fieldMap.put(entityKey)
            }
        }


        Map sortMap = new HashMap();
        if(!StringUtils.isEmpty(requestEntity.getSortFieldName())){
            sortMap.put("sortFieldName",requestEntity.getSortFieldName());
            sortMap.put("sortType",requestEntity.getSortType());
        }
        TimeInfo timeInfo =new TimeInfo();
        timeInfo.setStartTime(requestEntity.getStartTime());
        timeInfo.setEndTime(requestEntity.getEndTime());
        timeInfo.setFieldName(requestEntity.getTimeFieldName());
        ElasticPage elasticPage = getElasticEntityRepository().findEntity(requestEntity.getTableName(), fieldMap, timeInfo, pageInfo,sortMap);

        return ResponseResult.sucess(elasticPage.getTotalSize(),elasticPage.getResults());
    }

    private ArrayList<String> JSONArrayToArrayList(JSONArray jsonArray){
        ArrayList<String> listdata = null;
        if (jsonArray != null) {
            listdata = new ArrayList<String>();
            for (int i=0;i<jsonArray.size();i++){
                listdata.add(jsonArray.getString(i));
            }
        }
        return listdata;
    }

    private Object findEsEntity(){
        return null;
    }
}
